<#
.SYNOPSIS
This script replicates the "test voice routing / populate from user" in the Lync Control Panel, and is of
greatest benefit where your restrictions prevent you from using the Control Panel.

.DESCRIPTION
This script takes a username and dialled number, then retrieves the user's Dial Plan & Voice Policy. 
It passes the dialled number to "Test-CsVoiceUser" and if the call would be routed, displays the Usage, 
Route and Gateway that would route the call. If there are multiple gateways assigned to the route, 
only the first is shown.

The script requires Lync 2013 and the user must have permission to execute "Get-CsEffectivePolicy" and 
"Test-CsVoiceUser". Of the predefined RBAC roles, the minimum levels of membership are CsViewOnlyAdministrator
and one of CsHelpDesk or CsVoiceAdministrator.

.NOTES
Version             : 1.0
Date                :  3rd May 2014
Author              : Greig Sheridan
Revision History    :
			v1.0    : 3rd May 2014

.LINK
https://greiginsydney.com/Test-CallRouting-ps1

.EXAMPLE
.\Test-CallRouting.ps1
Description
-----------
With no command-line parameters you will be prompted for a user name and a dialled number.

.EXAMPLE
.\Test-CallRouting.ps1 -Identity "greig@contoso.com" -DialledNumber "049912345678"
Description
-----------
Will determine the user's Dial Plan, Voice Policy, Normalise the dialled number and provide details of the
gateway that would route the call.

.PARAMETER Identity
A standard Identity name, as you'd provide to the "Get-CsUser" commandlet.

.PARAMETER DialedNumber
A telephone number in the format as dialled by the user.

#>

[CmdletBinding(SupportsShouldProcess = $True)]
param(
[parameter(Position=0,ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true)]
[String]$Identity="",
[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true)]
[String]$DialedNumber=""
)

$Error.Clear()
$ErrorActionPreference = "stop"

#Setup some regionalisation for my American friends
# Let me know if any other countries (Locales) spell normalisation 'wrong' ;-)
$Locale = ""
try
{
	#This is new in Win 8:
	switch ((Get-WinSystemLocale).DisplayName)
	{
		{($_ -eq "English (Canada)") -or ($_ -eq "English (United States)")}
		{
			$Locale = "en-US"
		}
		default
		{
			$Locale = "TheRestOfUs"
		}
	}
}
catch {}

# This first command tests if you have a valid $PS session, and also tests the validity of the user account.
# (Get-csuser is one of the few commands available to EVERY standard RBAC group)
try
{
	$getUser = invoke-expression "get-csuser -identity ""$Identity"" -ea silentlycontinue" 
}
catch [System.Management.Automation.MethodInvocationException]
{
	write-warning "The Lync PowerShell module is not loaded, or your session has expired"
	#$_ | fl * -f
	exit
}
catch [System.Management.Automation.CommandNotFoundException]
{
	write-warning "The Lync PowerShell module is not loaded, or your session has expired"
	#$_ | fl * -f
	exit
}
catch 
{
	write-warning "Unhandled Error"
	$_
	#$_ | fl * -f
	exit
}

$ServerVersion = ""
#Just in case your RBAC won't let you get to this one
try
{
	$ServerVersion = get-csserverversion -ea silentlycontinue
}
catch {}
if ($ServerVersion -match "Server 2010")
{
	write-warning "The detected Lync Server version is 2010. This script requires the Lync 2013 commandlet ""Get-CsEffectivePolicy"""
	exit
}

if ($getUser -eq $null)
{
	write-warning "User `"$($Identity)`" does not exist in AD or is not enabled for Lync"
	exit
}

$sipaddress = [string]$getUser.SipAddress 
if ($getUser.EnterpriseVoiceEnabled -ne $true)
{
	write-warning "User `"$($Identity)`" is not enabled for Lync Enterprise Voice" 
	exit
}

try 
{
	$EffectiveUserPolicy = (Get-CsEffectivePolicy -Identity $Identity)
	$vp = [string]($EffectiveUserPolicy.VoicePolicy)
	$dp = [string]($EffectiveUserPolicy.LocationProfile)
}
catch [System.Management.Automation.CommandNotFoundException]
{
	write-warning "You have no access to the required Lync 2013 'Get-CsEffectivePolicy' commandlet. (Try CsViewOnlyAdministrator)" 
	exit
}
catch 
{
	write-warning "Unhandled Error"
	$_
	#$_ | fl * -f
	exit
}

$dialplan = [string](get-csdialplan -identity $dp).identity	#This step ensures we get "Site:Lync2013RTM" and not "Site:4"
$voicepolicy = [string](get-csvoicepolicy -identity $vp).identity	#This step ensures we get "Site:Lync2013RTM" and not "Site:4"

#Will RBAC let us do this?
try
{
	$callpath = invoke-expression "Test-CsVoiceUser -DialedNumber ""$DialedNumber"" -SipUri ""$sipaddress"" | select MatchingUsage,TranslatedNumber,MatchingRule,FirstMatchingRoute"
}
catch [System.Management.Automation.CommandNotFoundException]
{
	write-warning "You have no access to the required 'Test-CsVoiceUser' commandlet. (Try CsHelpDesk or CsVoiceAdministrator)"
	exit
}

$translatedNumber = $callpath.TranslatedNumber

if ($callpath.MatchingRule -eq $null) 
{
	if ($Locale -eq "en-US")
	{
		write-warning "The dialed number was not normalized by the dial plan" 
		$translatedNumber = ""
	}
	else
	{
		write-warning "The dialled number was not normalised by the dial plan" 
		$translatedNumber = ""
	}
}

$usage = $callpath.MatchingUsage
try
{
	#This will error if we couldn't route the call
	$firstRoute = $callPath | select -ExpandProperty FirstMatchingRoute
} 
catch
{	
	$firstRoute = ""
}

switch (($firstRoute.pstngatewaylist).Count)
{
	{($_ -eq "0") -or ($_ -eq $null)}
	{
		write-warning "The call cannot be routed" 
	}
	1
	{
		$trunk = [string]($firstRoute.pstngatewaylist[0]) #Get the FIRST gateway in the list (just in case there are multiples)
		$trunkname = get-cstrunk -identity $trunk
	}
	default
	{
		write-warning "More than one trunk was returned - only the first is shown" 
		$trunk = [string]($firstRoute.pstngatewaylist[0]) #Get the FIRST gateway in the list (just in case there are multiples)
		$trunkname = get-cstrunk -identity $trunk
	}
}

#Now create a custom object so we can output all this flexibly to screen or pipeline:
$Output = new-object PSObject
$Output | Add-Member NoteProperty -Name "SipAddress" -Value $sipaddress
$Output | Add-Member NoteProperty -Name "DialedNumber" -Value $dialedNumber
$Output | Add-Member NoteProperty -Name "DialPlan" -Value $dialplan
$Output | Add-Member NoteProperty -Name "VoicePolicy" -Value $voicepolicy
$Output | Add-Member NoteProperty -Name "Translated" -Value $translatedNumber
$Output | Add-Member NoteProperty -Name "Usage" -Value $usage
$Output | Add-Member NoteProperty -Name "SelectedRoute" -Value $firstRoute.Name
$Output | Add-Member NoteProperty -Name "FirstTrunk" -Value $trunkname.Identity
write-output $Output

#Code signing certificate kindly provided by Digicert:

# SIG # Begin signature block
# MIIfXwYJKoZIhvcNAQcCoIIfUDCCH0wCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUd+fheczzgLjCeRQyAbuFFCJQ
# RQigghqRMIIGajCCBVKgAwIBAgIQA5/t7ct5W43tMgyJGfA2iTANBgkqhkiG9w0B
# AQUFADBiMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSEwHwYDVQQDExhEaWdpQ2VydCBBc3N1cmVk
# IElEIENBLTEwHhcNMTMwNTIxMDAwMDAwWhcNMTQwNjA0MDAwMDAwWjBHMQswCQYD
# VQQGEwJVUzERMA8GA1UEChMIRGlnaUNlcnQxJTAjBgNVBAMTHERpZ2lDZXJ0IFRp
# bWVzdGFtcCBSZXNwb25kZXIwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIB
# AQC6aUqBTW+lFBaqis1nvku/xmmPWBzgeegenVgmmNpc1Hyj+dsrjBI2w/z5ZAax
# u8KomAoXDeGV60C065ZtmL+mj3nPvIqSe22cGAZR2KUYUzIBJxlh6IRB38bw6Mr+
# d61f2J57jGBvhVxGvWvnD4DO5wPDfDHPt2VVxvvgmQjkc1r7l9rQTL60tsYPfyaS
# qbj8OO605DqkSNBM6qlGJ1vPkhGTnBan/tKtHyLFHqzBce+8StsBCUTfmBwtZ7qo
# igMzyVG19wJNCaRN/oBexddFw30IqgEzzDPYTzAW5P8iMi7rfjvw+R4y65Ul0vL+
# bVSEutXl1NHdG6+9WXuUhTABAgMBAAGjggM1MIIDMTAOBgNVHQ8BAf8EBAMCB4Aw
# DAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcDCDCCAb8GA1UdIASC
# AbYwggGyMIIBoQYJYIZIAYb9bAcBMIIBkjAoBggrBgEFBQcCARYcaHR0cHM6Ly93
# d3cuZGlnaWNlcnQuY29tL0NQUzCCAWQGCCsGAQUFBwICMIIBVh6CAVIAQQBuAHkA
# IAB1AHMAZQAgAG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkAZgBpAGMAYQB0AGUA
# IABjAG8AbgBzAHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAAdABhAG4AYwBlACAA
# bwBmACAAdABoAGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAALwBDAFAAUwAgAGEA
# bgBkACAAdABoAGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIAdAB5ACAAQQBnAHIA
# ZQBlAG0AZQBuAHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQAIABsAGkAYQBiAGkA
# bABpAHQAeQAgAGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIAcABvAHIAYQB0AGUA
# ZAAgAGgAZQByAGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUAbgBjAGUALjALBglg
# hkgBhv1sAxUwHwYDVR0jBBgwFoAUFQASKxOYspkH7R7for5XDStnAs0wHQYDVR0O
# BBYEFGMvyd95knu1I8q74aTuM37j4p36MH0GA1UdHwR2MHQwOKA2oDSGMmh0dHA6
# Ly9jcmwzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRENBLTEuY3JsMDig
# NqA0hjJodHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURD
# QS0xLmNybDB3BggrBgEFBQcBAQRrMGkwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3Nw
# LmRpZ2ljZXJ0LmNvbTBBBggrBgEFBQcwAoY1aHR0cDovL2NhY2VydHMuZGlnaWNl
# cnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEQ0EtMS5jcnQwDQYJKoZIhvcNAQEFBQAD
# ggEBAKt0vUAATHYVJVc90xwD/31FyEUSZucoZWDY3zuz+g3BrDOP9IG5YfGd+5hV
# 195HQ7qAPfFIzD9nMFYfzvTQTIS9h6SexeEPqAZd0C9uXtwZ6PCH6uBOrz1sII5z
# b37WhxjghtOa/J7qjHLpQQ+4cbU4LPgpstUcop0b7F8quNw3IOHLu/DQbGyls8uf
# SvZU4yY0PS64wSsct/bDPf7RLR5Q9JTI+P3uc9tJtRv09f+lkME5FBvY7XEbapj7
# +kCaRKkpDlVeeLi3pIPDcAHwZkDlrnk04StNA6Et5ttUYhjt1QmLoqrWDMhPGr6Z
# JXhpmYnUWYne34jw02dedKWdpkQwggajMIIFi6ADAgECAhAPqEkGFdcAoL4hdv3F
# 7G29MA0GCSqGSIb3DQEBBQUAMGUxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdp
# Q2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xJDAiBgNVBAMTG0Rp
# Z2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0xMTAyMTExMjAwMDBaFw0yNjAy
# MTAxMjAwMDBaMG8xCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMx
# GTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xLjAsBgNVBAMTJURpZ2lDZXJ0IEFz
# c3VyZWQgSUQgQ29kZSBTaWduaW5nIENBLTEwggEiMA0GCSqGSIb3DQEBAQUAA4IB
# DwAwggEKAoIBAQCcfPmgjwrKiUtTmjzsGSJ/DMv3SETQPyJumk/6zt/G0ySR/6hS
# k+dy+PFGhpTFqxf0eH/Ler6QJhx8Uy/lg+e7agUozKAXEUsYIPO3vfLcy7iGQEUf
# T/k5mNM7629ppFwBLrFm6aa43Abero1i/kQngqkDw/7mJguTSXHlOG1O/oBcZ3e1
# 1W9mZJRru4hJaNjR9H4hwebFHsnglrgJlflLnq7MMb1qWkKnxAVHfWAr2aFdvftW
# k+8b/HL53z4y/d0qLDJG2l5jvNC4y0wQNfxQX6xDRHz+hERQtIwqPXQM9HqLckvg
# VrUTtmPpP05JI+cGFvAlqwH4KEHmx9RkO12rAgMBAAGjggNDMIIDPzAOBgNVHQ8B
# Af8EBAMCAYYwEwYDVR0lBAwwCgYIKwYBBQUHAwMwggHDBgNVHSAEggG6MIIBtjCC
# AbIGCGCGSAGG/WwDMIIBpDA6BggrBgEFBQcCARYuaHR0cDovL3d3dy5kaWdpY2Vy
# dC5jb20vc3NsLWNwcy1yZXBvc2l0b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIBVh6C
# AVIAQQBuAHkAIAB1AHMAZQAgAG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkAZgBp
# AGMAYQB0AGUAIABjAG8AbgBzAHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAAdABh
# AG4AYwBlACAAbwBmACAAdABoAGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAALwBD
# AFAAUwAgAGEAbgBkACAAdABoAGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIAdAB5
# ACAAQQBnAHIAZQBlAG0AZQBuAHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQAIABs
# AGkAYQBiAGkAbABpAHQAeQAgAGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIAcABv
# AHIAYQB0AGUAZAAgAGgAZQByAGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUAbgBj
# AGUALjASBgNVHRMBAf8ECDAGAQH/AgEAMHkGCCsGAQUFBwEBBG0wazAkBggrBgEF
# BQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEMGCCsGAQUFBzAChjdodHRw
# Oi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0Eu
# Y3J0MIGBBgNVHR8EejB4MDqgOKA2hjRodHRwOi8vY3JsMy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsMDqgOKA2hjRodHRwOi8vY3JsNC5k
# aWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsMB0GA1UdDgQW
# BBR7aM4pqsAXvkl64eU/1qf3RY81MjAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYun
# pyGd823IDzANBgkqhkiG9w0BAQUFAAOCAQEAe3IdZP+IyDrBt+nnqcSHu9uUkteQ
# WTP6K4feqFuAJT8Tj5uDG3xDxOaM3zk+wxXssNo7ISV7JMFyXbhHkYETRvqcP2pR
# ON60Jcvwq9/FKAFUeRBGJNE4DyahYZBNur0o5j/xxKqb9to1U0/J8j3TbNwj7aqg
# TWcJ8zqAPTz7NkyQ53ak3fI6v1Y1L6JMZejg1NrRx8iRai0jTzc7GZQY1NWcEDzV
# sRwZ/4/Ia5ue+K6cmZZ40c2cURVbQiZyWo0KSiOSQOiG3iLCkzrUm2im3yl/Brk8
# Dr2fxIacgkdCcTKGCZlyCXlLnXFp9UH/fzl3ZPGEjb6LHrJ9aKOlkLEM/zCCBqcw
# ggWPoAMCAQICEAL1iZyoL1yF+am1esglnfUwDQYJKoZIhvcNAQEFBQAwbzELMAkG
# A1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRp
# Z2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNlcnQgQXNzdXJlZCBJRCBDb2RlIFNp
# Z25pbmcgQ0EtMTAeFw0xNDAxMTAwMDAwMDBaFw0xNTAxMTQxMjAwMDBaMHMxCzAJ
# BgNVBAYTAkFVMRgwFgYDVQQIEw9OZXcgU291dGggV2FsZXMxGDAWBgNVBAcTD1Bl
# dGVyc2hhbSBOb3J0aDEXMBUGA1UEChMOR3JlaWcgU2hlcmlkYW4xFzAVBgNVBAMT
# DkdyZWlnIFNoZXJpZGFuMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
# 5eB+AwrP4k2/C3qOD3VNoBoGN+IMMkGfpB/YwrbWx2hRFR5L+XZlA0468ijkY3Fp
# f7gpVqKknzwjyJbPGmVszOrDbH2OJnoOpfOpCKCu0apVh2ESKchPUQYtwQV+B/Eq
# JykWDYhEGzSbdduQ03wAA3pWkI3KqFhomEOlUhvn4YydOflFHy/Ysr20rVpju7yC
# 1Qjo58JPp31w5xxrP4KVE2aRnUi0Fan4jiqUbkaZhdYL2Bj/1eYOy6dxFfmSSrah
# K7Pj+8MTVD2ANQzH7r/8tUtXXopznHdI7ENqRia7mPyhuCj/HMRUi41TfGncibV4
# Fzcyiw4x6cCnVMM3D4c44QIDAQABo4IDOTCCAzUwHwYDVR0jBBgwFoAUe2jOKarA
# F75JeuHlP9an90WPNTIwHQYDVR0OBBYEFCjhLo4ioIIdfp0nv8ERTCMoB2BpMA4G
# A1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBzBgNVHR8EbDBqMDOg
# MaAvhi1odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vYXNzdXJlZC1jcy0yMDExYS5j
# cmwwM6AxoC+GLWh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9hc3N1cmVkLWNzLTIw
# MTFhLmNybDCCAcQGA1UdIASCAbswggG3MIIBswYJYIZIAYb9bAMBMIIBpDA6Bggr
# BgEFBQcCARYuaHR0cDovL3d3dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBvc2l0
# b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAgAG8A
# ZgAgAHQAaABpAHMAIABDAGUAcgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBzAHQA
# aQB0AHUAdABlAHMAIABhAGMAYwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABoAGUA
# IABEAGkAZwBpAEMAZQByAHQAIABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABoAGUA
# IABSAGUAbAB5AGkAbgBnACAAUABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBuAHQA
# IAB3AGgAaQBjAGgAIABsAGkAbQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAgAGEA
# bgBkACAAYQByAGUAIABpAG4AYwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQByAGUA
# aQBuACAAYgB5ACAAcgBlAGYAZQByAGUAbgBjAGUALjCBggYIKwYBBQUHAQEEdjB0
# MCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wTAYIKwYBBQUH
# MAKGQGh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJ
# RENvZGVTaWduaW5nQ0EtMS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQUF
# AAOCAQEAPqlEkuWrHzYJ+9vALPViHOhpqLIAG4c25ncYggtQpn2zO/zd7JVC0g2E
# wzprd2nDNeuHhFhMrFpuc4cv+5Z+LgFcewY/D6F6n4sjm6A5T43b4UY8H1aJBPzw
# hUyZtQSHU2P90iVas+1D+aYBXihOxS1JnUk/fVg5llX6v8UW4u5aHiUxe3RLs40q
# dqoBLooazXKQDajIUrYvfOEN5o2Pp1Ehn8UwwEeSLJU7IQaAWW4AmCzHF/cnDWJJ
# 1cSlFj7oifNPaRtQ0SHzk4+9QW8/kwMyCbTAOQ2rH4qPnQpCCq2qaznPuDyjA6RI
# Z9B8/mD3zXNHDXgdpSXwRmmSvcdQzzCCBs0wggW1oAMCAQICEAb9+QOWA63qAArr
# Pye7uhswDQYJKoZIhvcNAQEFBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERp
# Z2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UEAxMb
# RGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTA2MTExMDAwMDAwMFoXDTIx
# MTExMDAwMDAwMFowYjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IElu
# YzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8GA1UEAxMYRGlnaUNlcnQg
# QXNzdXJlZCBJRCBDQS0xMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
# 6IItmfnKwkKVpYBzQHDSnlZUXKnE0kEGj8kz/E1FkVyBn+0snPgWWd+etSQVwpi5
# tHdJ3InECtqvy15r7a2wcTHrzzpADEZNk+yLejYIA6sMNP4YSYL+x8cxSIB8HqIP
# kg5QycaH6zY/2DDD/6b3+6LNb3Mj/qxWBZDwMiEWicZwiPkFl32jx0PdAug7Pe2x
# QaPtP77blUjE7h6z8rwMK5nQxl0SQoHhg26Ccz8mSxSQrllmCsSNvtLOBq6thG9I
# hJtPQLnxTPKvmPv2zkBdXPao8S+v7Iki8msYZbHBc63X8djPHgp0XEK4aH631XcK
# J1Z8D2KkPzIUYJX9BwSiCQIDAQABo4IDejCCA3YwDgYDVR0PAQH/BAQDAgGGMDsG
# A1UdJQQ0MDIGCCsGAQUFBwMBBggrBgEFBQcDAgYIKwYBBQUHAwMGCCsGAQUFBwME
# BggrBgEFBQcDCDCCAdIGA1UdIASCAckwggHFMIIBtAYKYIZIAYb9bAABBDCCAaQw
# OgYIKwYBBQUHAgEWLmh0dHA6Ly93d3cuZGlnaWNlcnQuY29tL3NzbC1jcHMtcmVw
# b3NpdG9yeS5odG0wggFkBggrBgEFBQcCAjCCAVYeggFSAEEAbgB5ACAAdQBzAGUA
# IABvAGYAIAB0AGgAaQBzACAAQwBlAHIAdABpAGYAaQBjAGEAdABlACAAYwBvAG4A
# cwB0AGkAdAB1AHQAZQBzACAAYQBjAGMAZQBwAHQAYQBuAGMAZQAgAG8AZgAgAHQA
# aABlACAARABpAGcAaQBDAGUAcgB0ACAAQwBQAC8AQwBQAFMAIABhAG4AZAAgAHQA
# aABlACAAUgBlAGwAeQBpAG4AZwAgAFAAYQByAHQAeQAgAEEAZwByAGUAZQBtAGUA
# bgB0ACAAdwBoAGkAYwBoACAAbABpAG0AaQB0ACAAbABpAGEAYgBpAGwAaQB0AHkA
# IABhAG4AZAAgAGEAcgBlACAAaQBuAGMAbwByAHAAbwByAGEAdABlAGQAIABoAGUA
# cgBlAGkAbgAgAGIAeQAgAHIAZQBmAGUAcgBlAG4AYwBlAC4wCwYJYIZIAYb9bAMV
# MBIGA1UdEwEB/wQIMAYBAf8CAQAweQYIKwYBBQUHAQEEbTBrMCQGCCsGAQUFBzAB
# hhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQwYIKwYBBQUHMAKGN2h0dHA6Ly9j
# YWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcnQw
# gYEGA1UdHwR6MHgwOqA4oDaGNGh0dHA6Ly9jcmwzLmRpZ2ljZXJ0LmNvbS9EaWdp
# Q2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwOqA4oDaGNGh0dHA6Ly9jcmw0LmRpZ2lj
# ZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwHQYDVR0OBBYEFBUA
# EisTmLKZB+0e36K+Vw0rZwLNMB8GA1UdIwQYMBaAFEXroq/0ksuCMS1Ri6enIZ3z
# bcgPMA0GCSqGSIb3DQEBBQUAA4IBAQBGUD7Jtygkpzgdtlspr1LPUukxR6tWXHvV
# DQtBs+/sdR90OPKyXGGinJXDUOSCuSPRujqGcq04eKx1XRcXNHJHhZRW0eu7NoR3
# zCSl8wQZVann4+erYs37iy2QwsDStZS9Xk+xBdIOPRqpFFumhjFiqKgz5Js5p8T1
# zh14dpQlc+Qqq8+cdkvtX8JLFuRLcEwAiR78xXm8TBJX/l/hHrwCXaj++wc4Tw3G
# XZG5D2dFzdaD7eeSDY2xaYxP+1ngIw/Sqq4AfO6cQg7PkdcntxbuD8O9fAqg7iwI
# VYUiuOsYGk38KiGtSTGDR5V3cdyxG0tLHBCcdxTBnU8vWpUIKRAmMYIEODCCBDQC
# AQEwgYMwbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcG
# A1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNlcnQgQXNzdXJl
# ZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMQIQAvWJnKgvXIX5qbV6yCWd9TAJBgUrDgMC
# GgUAoHgwGAYKKwYBBAGCNwIBDDEKMAigAoAAoQKAADAZBgkqhkiG9w0BCQMxDAYK
# KwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAjBgkqhkiG
# 9w0BCQQxFgQUzAvBGIqHhXsNJutZpRtSuuDg1CowDQYJKoZIhvcNAQEBBQAEggEA
# HE8Nrcd8k3+GcWB1l6STauyt5My4RzIMJH1a8Ul5k6SW0rilrByT5DjCukyyeUa3
# UJtEUUa/vUEW4p/QMiKBs+yQUTnEe3vrONfacu2koF2Afb+c6e1ViH1PxL2BWYFR
# c9zl+8phLkXmQO/3M5F4VKN4XKFn6X0nrvzIZpZNuJGliiRvoOlPjmbpwXbHnucu
# VSKkLtRjKAVYQuFfoCBK8KhWBs0oz8KFh/g5iEUpQWnt4MEmvOYrsY7oJygt8WEd
# TfSIGpd3YmwGzLft45eMgZBbuNFIhR1DRHKOsjYgMUTcI13PwOQszet2Mlix5crf
# 5uvHN62Cj6tP9TR6ro6256GCAg8wggILBgkqhkiG9w0BCQYxggH8MIIB+AIBATB2
# MGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsT
# EHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lDZXJ0IEFzc3VyZWQgSUQg
# Q0EtMQIQA5/t7ct5W43tMgyJGfA2iTAJBgUrDgMCGgUAoF0wGAYJKoZIhvcNAQkD
# MQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMTQwNTAzMDQ0NjM1WjAjBgkq
# hkiG9w0BCQQxFgQUYtxeqZ1rcYDrD60PQKbW7g7ySgcwDQYJKoZIhvcNAQEBBQAE
# ggEAfju6eNU3Bu98d8y13m99wMcX3oSDi1q5u4Z3z+3wtghZtqvwNuYS5he+uXAT
# Vs3YaCSTwgJcQQZ4OcAfjq/55V/l0xC1otl+UPHYR6n8FCTWindQoZ/zhRtmQWXH
# 1zgwVuUFDJshHpi3Ke3QYzZ1o/n3G+5SVGOQ19KYge8dUuW8TdwAgkb52gSV3rc4
# zY3EZoWI1UaUVLRjTJ2NhrsWv03rhLFEwF9iAq1eeJXS/AGeg+dgZcP2ng6rkcPv
# vIT/MYItLyyYfdES0J3I5DIPRIY3Z5qvCPK4ZyJ5oMl6dkGAq4kUt71gJENeU3mG
# +7C1wL0cpsH/9bOnLHohEL+ZjQ==
# SIG # End signature block
